@@ -35,6 +35,10 @@
     } else {
         subsetNArm <- subset[!(subset %in% c(NAyx, NAz))]
     }
+    ### report by Kevin Ummel: _all_ obs being missing lead to
+    ### subset being ignored completely
+    if (length(subsetNArm) == 0) 
+        return(list(statistic = NA, p.value = NA))
 
     return(.ctree_test_1d(data = data, j = j, Y = Y, subset = subsetNArm,
                           weights = weights, SPLITONLY = SPLITONLY, ctrl = ctrl))
@@ -210,7 +214,9 @@
     }
 
     ### check if either X or Y were unique
-    if (all(lev$Variance < ctrl$tol)) {
+    vr <- lev$Variance
+    vr[is.na(vr)] <- 0
+    if (all(vr < ctrl$tol)) {
         if (SPLITONLY) return(NULL)
         return(list(statistic = NA, p.value = NA))
     }
@@ -332,6 +338,7 @@ ctree_control <- function
     minbucket = 7L,
     minprob = 0.01,
     stump = FALSE,
+    maxvar = Inf,
     lookahead = FALSE,	### try trafo() for daugther nodes before implementing the split
     MIA = FALSE,	### DOI: 10.1016/j.patrec.2008.01.010
     nresample = 9999L,
@@ -378,7 +385,7 @@ ctree_control <- function
                                       "statistic", "p.value"),
                      logmincriterion = logmincriterion, minsplit = minsplit,
                      minbucket = minbucket, minprob = minprob,
-                     nmax = nmax, stump = stump, lookahead = lookahead,
+                     nmax = nmax, maxvar = maxvar, stump = stump, lookahead = lookahead,
                      mtry = mtry, maxdepth = maxdepth, multiway = multiway,
                      splittry = splittry, maxsurrogate = maxsurrogate,
                      numsurrogate = numsurrogate,
