@@ -73,7 +73,7 @@ get_paths <- function(obj, i) {
   return(x)
 }
 
-"nodeids<-.constparty" <- function(obj, value) {
+"nodeids<-.constparty" <- "nodeids<-.modelparty" <- function(obj, value) {
 
   id0 <- nodeids(obj)
   cls <- class(obj)
@@ -112,5 +112,5 @@ get_paths <- function(obj, i) {
   indx
 }
 
-.rfweights <- function(fdata, fnewdata, rw)
-    w <- .Call(R_rfweights, fdata, fnewdata, rw)
+.rfweights <- function(fdata, fnewdata, rw, scale)
+    w <- .Call(R_rfweights, fdata, fnewdata, rw, scale)
